"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.InteractiveOverlay = void 0;
const react_1 = __importDefault(require("react"));
const styled_components_1 = __importStar(require("styled-components"));
const overlayStyles = styled_components_1.css `
  &::before {
    // TODO: border-radius;

    content: '';
    position: absolute;
    z-index: 0;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;

    // TODO: Ideally we have specific radius variants, or a way to inherit border
    // radius from the item wrapping the ::before.
    border-radius: ${({ radius }) => (radius ? `${radius}px` : `initial`)};
  }

  &:focus-visible {
    outline: none;

    &::before {
      outline: #ac9cff solid 2px;
      outline-offset: -2px;
    }
  }

  // Hover styles should be part of the wrapped children
`;
const StyledAnchor = styled_components_1.default.a `
  // Reset anchor styles
  // The actual styling of the element should happen in the children
  display: flex;
  text-decoration: none;
  color: inherit;
  font-family: inherit;

  ${overlayStyles}
`;
const StyledButton = styled_components_1.default.button `
  // Reset button styles
  // The actual styling of the element should happen in the children
  display: flex;
  border: none;
  padding: 0;
  background: transparent;
  color: inherit;
  font-family: inherit;
  cursor: pointer;

  &:disabled {
    cursor: not-allowed;
  }

  ${overlayStyles}
`;
const StyledItem = styled_components_1.default(({ children, className: styledClassName }) => {
    if (react_1.default.Children.count(children) > 1) {
        throw new Error('The StyledOverlay.Item component can only contain one child.');
    }
    const augmentedChildren = react_1.default.Children.map(children, child => {
        if (react_1.default.isValidElement(child)) {
            // TODO: fix ts-ignore "Property 'className' does not exist on type 'unknown'."
            // @ts-ignore
            return react_1.default.cloneElement(child, { className: styledClassName });
        }
        return child;
    });
    if (augmentedChildren.length > 0) {
        // eslint-disable-next-line react/jsx-no-useless-fragment
        return react_1.default.createElement(react_1.default.Fragment, null, augmentedChildren);
    }
    return children;
}) `
  ${overlayStyles}
`;
/**
 * The styled `Wrapper` is used to wrap content inside a clickable (button or anchor) area while ensuring
 * semantic html.
 *
 * Inspired by the Chakra LinkOverlay: https://chakra-ui.com/docs/navigation/link-overlay
 * More information on nested links: https://www.sarasoueidan.com/blog/nested-links
 */
const Wrapper = styled_components_1.default(({ children, isElement = false, className: styledClassName, }) => {
    if (react_1.default.Children.count(children) > 1) {
        throw new Error('The InteractiveOverlay component can only contain one child.');
    }
    if (!isElement) {
        const augmentedChildren = react_1.default.Children.map(children, child => {
            if (react_1.default.isValidElement(child)) {
                // TODO: fix ts-ignore "Property 'className' does not exist on type 'unknown'."
                // @ts-ignore
                return react_1.default.cloneElement(child, { className: styledClassName });
            }
            return child;
        });
        if (augmentedChildren.length > 0) {
            // eslint-disable-next-line react/jsx-no-useless-fragment
            return react_1.default.createElement(react_1.default.Fragment, null, augmentedChildren);
        }
    }
    return react_1.default.createElement("div", { className: styledClassName }, children);
}) `
  position: relative;

  // Elevate the anchors with href and buttons (except StyledItem) up
  // TODO: Is there an easier way to target just one class instead of having to chain :not?
  & a[href]:not(${StyledItem}):not(${StyledAnchor}),
  button:not(${StyledItem}):not(${StyledButton}) {
    position: relative;
    z-index: 1;
  }
`;
// Wrapper around Wrapper to ensure compound component works. There seems to be an issue
// with extending `styled` component with `.Item` etcetera.
const InteractiveOverlay = ({ children, isElement, }) => {
    return react_1.default.createElement(Wrapper, { isElement: isElement }, children);
};
exports.InteractiveOverlay = InteractiveOverlay;
exports.InteractiveOverlay.Item = StyledItem;
exports.InteractiveOverlay.Button = StyledButton;
exports.InteractiveOverlay.Anchor = StyledAnchor;
